import cv2
from tqdm import tqdm
import numpy as np
import scipy.io as sio
import matplotlib.pyplot as plt
import os

def generate_ROI(folder_path, threshold_value=85):
    # 获取文件夹中的所有文件
    files = [f for f in os.listdir(folder_path) if f.endswith(('.bmp', '.png', '.jpg'))]
    files.sort()  # 确保文件的顺序
    images = []

    # 读取后四张图像/高频
    for file in files[-4:]:
        img_path = os.path.join(folder_path, file)
        img = cv2.imread(img_path, cv2.IMREAD_GRAYSCALE)
        if img is None:
            raise ValueError("Failed to read image from path: " + img_path)
        images.append(img)

    # 确保所有图像尺寸相同
    if len(set([img.shape for img in images])) != 1:
        raise ValueError("All images must have the same dimensions")

    # 将四张图像的像素值相加
    result_img = cv2.add(images[0], images[1])
    for img in images[2:]:
        result_img = cv2.add(result_img, img)

    # 使用阈值处理：大于85的地方赋值为255（白色），其他赋值为0（黑色）
    _, threshold_img = cv2.threshold(result_img, threshold_value, 255, cv2.THRESH_BINARY)

    return threshold_img
    # 保存处理后的图像
    # output_path = os.path.join(folder_path, 'white.bmp')

    # cv2.imwrite(output_path, threshold_img)
    # print(f"Image saved successfully at {output_path}")